# -*- coding: UTF-8 -*-
# Name: long_processing_task.py
# Porpose: Console to show logging messages during processing
# Compatibility: Python3, wxPython4 Phoenix
# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Feb.03.2021 *PEP8 compatible*
#########################################################
# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################
from __future__ import unicode_literals
import wx
import os
from pubsub import pub
from videomass3.vdms_io.make_filelog import write_log
from videomass3.vdms_threads.ydl_pylibdownloader import Ydl_DL_Pylib
from videomass3.vdms_threads.ydl_executable import Ydl_DL_Exec
from videomass3.vdms_threads.one_pass import OnePass
from videomass3.vdms_threads.two_pass import TwoPass
from videomass3.vdms_threads.two_pass_EBU import Loudnorm
from videomass3.vdms_threads.picture_exporting import PicturesFromVideo
from videomass3.vdms_threads.video_stabilization import VidStab
from videomass3.vdms_threads.concat_demuxer import Concat_Demuxer
from videomass3.vdms_utils.utils import get_milliseconds


def pairwise(iterable):
    """
    Return a zip object from iterable.
    This function is used by the update_display method.
    ----
    USE:

    after splitting ffmpeg's progress strings such as:
    output = "frame= 1178 fps=155 q=29.0 size=    2072kB time=00:00:39.02
              bitrate= 435.0kbits/s speed=5.15x  "
    in a list as:
    iterable = ['frame', '1178', 'fps', '155', 'q', '29.0', 'size', '2072kB',
                'time', '00:00:39.02', 'bitrate', '435.0kbits/s', speed',
                '5.15x']
    for x, y in pairwise(iterable):
        print(x,y)

    <https://stackoverflow.com/questions/5389507/iterating-over-every-
    two-elements-in-a-list>

    """
    a = iter(iterable)  # list_iterator object
    return zip(a, a)  # zip object pairs from list iterable object
# ----------------------------------------------------------------------#


class Logging_Console(wx.Panel):
    """
    displays a text control for the output logging, a progress bar
    and a progressive percentage text label. This panel is used
    in combination with separated threads for long processing tasks.
    It also implements stop and close buttons to stop the current
    process and close the panel at the end.

    """
    # get videomass wx.App attribute
    get = wx.GetApp()
    OS = get.OS
    LOGDIR = get.LOGdir

    # used msg on text
    MSG_done = _('[Videomass]: SUCCESS !\n')
    MSG_failed = _('[Videomass]: FAILED !\n')
    MSG_taskfailed = _('\n[Videomass]: Sorry, task failed !\n')
    MSG_interrupted = _('\n[Videomass]: Interrupted Process !\n')
    MSG_completed = _('\n[Videomass]: Successfully completed !\n')
    MSG_unfinished = _('\n[Videomass]: completed, but not everything '
                       'was successful.\n')

    # Colors used in HTML
    if get.THEME in ('Breeze-Blues', 'Videomass-Colours'):
        BACKGROUND = '#11303eff'  # SOLARIZED background color for blues icons
        NORM_TEXT = '#FFFFFF'  # WHITE for title or URL in progress
        TEXT_2 = '#959595'  # GREY for all other text messages
        ERROR = '#FF4A1B'  # ORANGE for error text messages
        WARN = '#dfb72f'  # YELLOW for warning text messages
        ERROR_2 = '#EA312D'  # LIGHTRED for errors 2
        SUCCESS = '#1EA41E'  # GREEN when it is successful

    elif get.THEME in ('Breeze-Blues', 'Breeze-Dark', 'Videomass-Dark'):
        BACKGROUND = '#1c2027ff'  # DARK_SLATE background color for dark theme
        NORM_TEXT = '#FFFFFF'  # WHITE for title or URL in progress
        TEXT_2 = '#959595'  # GREY for all other text messages
        ERROR = '#FF4A1B'  # ORANGE for error text messages
        WARN = '#dfb72f'  # YELLOW for warning text messages
        ERROR_2 = '#EA312D'  # LIGHTRED for errors 2
        SUCCESS = '#1EA41E'  # GREEN when it is successful

    else:
        BACKGROUND = '#e6e6faff'  # LAVENDER background color for light theme
        NORM_TEXT = '#1f1f1fff'  # BLACK for title or URL in progress
        TEXT_2 = '#778899ff'  # LIGHT_SLATE for all other text messages
        ERROR = '#d25c07'  # ORANGE for error text messages
        WARN = '#988313'  # YELLOW for warning text messages
        ERROR_2 = '#c8120b'  # LIGHTRED for errors 2
        SUCCESS = '#008000'  # DARK_GREEN when it is successful

    INFO = '#31BAA7'  # CYAN for info text messages
    DEBUG = '#3298FB'  # AZURE for debug messages
    FAILED = '#D21814'  # RED_DEEP if failed
    ABORT = '#A41EA4'  # VIOLET if the user stops the processes
    WHITE = '#fbf4f4'  # white for background status bar
    BLACK = '#060505'  # black for background status bar
    # YELLOW = '#C8B72F'  # for warning text messages
    # RED = '#ff0000ff'
    # ORANGE_DEEP = '#E92D15'
    # GREEN = '#008000'
    # DARK_GREEN = '#008000'
    # BLACK = '#242424'
    # ------------------------------------------------------------------#

    def __init__(self, parent):
        """
        In the 'previous' attribute is stored an ID string used to
        recover the previous panel from which the process is started.
        The 'logname' attribute is the name_of_panel.log file in which
        log messages will be written

        """
        self.parent = parent  # main frame
        self.PARENT_THREAD = None  # the instantiated thread
        self.ABORT = False  # if True set to abort current process
        self.ERROR = False  # if True, all the tasks was failed
        self.previus = None  # panel name from which it starts
        self.logname = None  # example: AV_conversions.log
        self.result = None  # result of the final process

        wx.Panel.__init__(self, parent=parent)
        """ Constructor """

        infolbl = _("Process log:")
        lbl = wx.StaticText(self, label=infolbl)
        if Logging_Console.OS != 'Darwin':
            lbl.SetLabelMarkup("<b>%s</b>" % infolbl)
        self.OutText = wx.TextCtrl(self, wx.ID_ANY, "",
                                   style=wx.TE_MULTILINE |
                                   wx.TE_READONLY |
                                   wx.TE_RICH2
                                   )
        self.ckbx_text = wx.CheckBox(self, wx.ID_ANY, (_("Suppress excess "
                                                         "output")))
        self.barProg = wx.Gauge(self, wx.ID_ANY, range=0)
        self.labPerc = wx.StaticText(self, label="")
        self.button_stop = wx.Button(self, wx.ID_STOP, _("Abort"))
        self.button_close = wx.Button(self, wx.ID_CLOSE, "")
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((0, 10))
        sizer.Add(lbl, 0, wx.ALL, 5)
        sizer.Add(self.OutText, 1, wx.EXPAND | wx.ALL, 5)
        sizer.Add(self.ckbx_text, 0, wx.ALL, 5)
        sizer.Add(self.barProg, 0, wx.EXPAND | wx.ALL, 5)
        sizer.Add(self.labPerc, 0, wx.ALL, 5)
        #grid = wx.GridSizer(1, 2, 0, 0)
        grid = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(grid, 0, flag=wx.ALIGN_RIGHT | wx.RIGHT, border=0)
        grid.Add(self.button_stop, 0, wx.EXPAND | wx.ALL, 5)
        grid.Add(self.button_close, 0, wx.EXPAND | wx.ALL, 5)
        # set_properties:
        self.OutText.SetBackgroundColour(Logging_Console.BACKGROUND)
        self.ckbx_text.SetToolTip(_('If activated, hides some '
                                    'output messages.'))
        self.button_stop.SetToolTip(_("Stops current process"))
        self.SetSizerAndFit(sizer)
        # bind
        self.Bind(wx.EVT_BUTTON, self.on_stop, self.button_stop)
        self.Bind(wx.EVT_BUTTON, self.on_close, self.button_close)

        # ------------------------------------------
        self.button_stop.Enable(True)
        self.button_close.Enable(False)

        pub.subscribe(self.youtubedl_from_import, "UPDATE_YDL_FROM_IMPORT_EVT")
        pub.subscribe(self.youtubedl_exec, "UPDATE_YDL_EXECUTABLE_EVT")
        pub.subscribe(self.update_display, "UPDATE_EVT")
        pub.subscribe(self.update_count, "COUNT_EVT")
        pub.subscribe(self.end_proc, "END_EVT")
    # ----------------------------------------------------------------------

    def topic_thread(self, panel, varargs, duration, time_seq):
        """
        Thread redirection
        varargs: type tuple data object
        duration: total duration or partial if set time_seq
        """
        self.previus = panel  # stores the panel from which it starts

        if varargs[0] == 'Viewing last log':
            self.button_stop.Enable(False)
            self.button_close.Enable(True)
            return

        self.OutText.Clear(), self.labPerc.SetLabel('')
        self.logname = varargs[8]  # example: Videomass_VideoConversion.log

        write_log(self.logname, Logging_Console.LOGDIR)  # set initial file LOG

        if varargs[0] == 'onepass':  # from Audio/Video Conv.
            self.PARENT_THREAD = OnePass(varargs, duration,
                                         self.logname, time_seq,
                                         )
        elif varargs[0] == 'twopass':  # from Video Conv.
            self.PARENT_THREAD = TwoPass(varargs, duration,
                                         self.logname, time_seq
                                         )
        elif varargs[0] == 'two pass EBU':  # from Audio/Video Conv.
            self.PARENT_THREAD = Loudnorm(varargs, duration,
                                          self.logname, time_seq
                                          )
        elif varargs[0] == 'savepictures':
            self.PARENT_THREAD = PicturesFromVideo(varargs, duration,
                                                   self.logname, time_seq
                                                   )
        elif varargs[0] == 'libvidstab':  # from Audio/Video Conv.
            self.PARENT_THREAD = VidStab(varargs, duration,
                                         self.logname, time_seq
                                         )
        elif varargs[0] == 'concat_demuxer':  # from Concatenation Demuxer
            self.PARENT_THREAD = Concat_Demuxer(varargs, duration,
                                                self.logname, time_seq
                                                )
        elif varargs[0] == 'youtube_dl python package':  # as import youtube_dl
            self.ckbx_text.Hide()
            self.PARENT_THREAD = Ydl_DL_Pylib(varargs, self.logname)

        elif varargs[0] == 'youtube-dl executable':  # as youtube-dl exec.
            self.ckbx_text.Hide()
            self.PARENT_THREAD = Ydl_DL_Exec(varargs, self.logname)
    # ----------------------------------------------------------------------

    def youtubedl_from_import(self, output, duration, status):
        """
        Receiving output messages from youtube_dl library via
        pubsub "UPDATE_YDL_FROM_IMPORT_EVT" .
        """
        if status == 'ERROR':
            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.ERROR))
            self.OutText.AppendText('%s\n' % output)
            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.FAILED))
            self.OutText.AppendText(Logging_Console.MSG_failed)
            self.result = 'failed'

        elif status == 'WARNING':
            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.WARN))
            self.OutText.AppendText('%s\n' % output)

        elif status == 'DEBUG':
            if '[download] Destination' in output:
                self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.DEBUG))
                self.OutText.AppendText('%s\n' % output)

            elif '[info]' in output:
                self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.INFO))
                self.OutText.AppendText('%s\n' % output)

            elif '[download]' not in output:
                self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.TEXT_2))
                self.OutText.AppendText('%s\n' % output)

                with open(os.path.join(Logging_Console.LOGDIR,
                                       self.logname), "a") as logerr:
                    logerr.write("[YOUTUBE_DL]: %s > %s\n" % (status, output))
        elif status == 'DOWNLOAD':
            self.labPerc.SetLabel("%s" % duration[0])
            self.barProg.SetValue(duration[1])

        elif status == 'FINISHED':
            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.TEXT_2))
            self.OutText.AppendText('%s\n' % duration)

        if status in ['ERROR', 'WARNING']:
            with open(os.path.join(Logging_Console.LOGDIR,
                                   self.logname), "a") as logerr:
                logerr.write("[YOUTUBE_DL]: %s\n" % (output))
    # ---------------------------------------------------------------------#

    def youtubedl_exec(self, output, duration, status):
        """
        Receiving output messages from youtube-dl command line execution
        via pubsub "UPDATE_YDL_EXECUTABLE_EVT" .

        """
        if not status == 0:  # error, exit status of the p.wait
            if output:
                if 'ERROR:' in output:
                    self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.ERROR))
                    self.OutText.AppendText('%s\n' % output)

            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.ERROR_2))
            self.OutText.AppendText(Logging_Console.MSG_failed)
            self.result = 'failed'
            return

        if '[download]' in output:  # ...in processing
            if 'Destination' not in output:
                try:
                    i = float(output.split()[1].split('%')[0])
                except ValueError:
                    self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.WARN))
                    self.OutText.AppendText(' %s' % output)
                else:
                    self.barProg.SetValue(i)
                    self.labPerc.SetLabel("%s" % output)
                    del output, duration

        else:  # append all others lines on the textctrl and log file
            if not self.ckbx_text.IsChecked():  # print the output
                if '[info]' in output:
                    self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.INFO))
                    self.OutText.AppendText(' %s' % output)
                elif 'WARNING:' in output:
                    self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.WARN))
                    self.OutText.AppendText(' %s' % output)
                elif 'ERROR:' not in output:
                    self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.TEXT_2))
                    self.OutText.AppendText(' %s' % output)

            with open(os.path.join(Logging_Console.LOGDIR,
                                   self.logname), "a") as logerr:
                logerr.write("[YOUTUBE-DL]: %s" % (output))
                # write a row error into file log

    # ---------------------------------------------------------------------#

    def update_display(self, output, duration, status):
        """
        Receive message from thread of the second loops process
        by wxCallafter and pubsub UPDATE_EVT.
        The received 'output' is parsed for calculate the bar
        progress value, percentage label and errors management.
        This method can be used even for non-loop threads.

        NOTE: During conversion the ffmpeg errors do not stop all
              others tasks, if an error occurred it will be marked
              with 'failed' but continue; if it has finished without
              errors it will be marked with 'done' on update_count
              method. Since not all ffmpeg messages are errors, sometimes
              it happens to see more output marked with yellow color.

        This strategy consists first of capturing all the output and
        marking it in yellow, then in capturing the error if present,
        but exiting immediately after the function.

        """
        # if self.ckbx_text.IsChecked(): #ffmpeg output messages in real time:
        #    self.OutText.AppendText(output)

        if not status == 0:  # error, exit status of the p.wait
            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.ERROR_2))
            self.OutText.AppendText(Logging_Console.MSG_failed)
            self.result = 'failed'
            return  # must be return here

        if 'time=' in output:  # ...in processing
            i = output.index('time=')+5
            pos = output[i:].split()[0]
            ms = get_milliseconds(pos)

            if ms > duration:
                self.barProg.SetValue(duration)
            else:
                self.barProg.SetValue(ms)

            percentage = round((ms / duration) * 100 if duration != 0 else 100)
            out = [a for a in "=".join(output.split()).split('=') if a]
            ffprog = []
            for x, y in pairwise(out):
                ffprog.append("%s: %s | " % (x, y))
            self.labPerc.SetLabel("Processing... %s%% | %s" %
                                  (str(int(percentage)), "".join(ffprog)))
            del output, duration

        else:  # append all others lines on the textctrl and log file
            if not self.ckbx_text.IsChecked():  # not print the output
                if [x for x in ('info', 'Info') if x in output]:
                    self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.INFO))
                    self.OutText.AppendText('%s' % output)

                elif [x for x in ('Failed', 'failed', 'Error', 'error')
                      if x in output]:
                    self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.ERROR))
                    self.OutText.AppendText('%s' % output)

                elif [x for x in ('warning', 'Warning') if x in output]:
                    self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.WARN))
                    self.OutText.AppendText('%s' % output)

                else:
                    self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.TEXT_2))
                    self.OutText.AppendText('%s' % output)

            with open(os.path.join(Logging_Console.LOGDIR,
                                   self.logname), "a") as logerr:
                logerr.write("[FFMPEG]: %s" % (output))
                # write a row error into file log
    # ----------------------------------------------------------------------

    def update_count(self, count, duration, fname, end):
        """
        Receive message from first 'for' loop in the thread process.
        This method can be used even for non-loop threads.

        """
        if end == 'ok':
            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.SUCCESS))
            self.OutText.AppendText(Logging_Console.MSG_done)
            if self.labPerc.GetLabel()[1] != '100%':
                newlab = self.labPerc.GetLabel().split()
                newlab[1] = '100%'
                self.labPerc.SetLabel(" ".join(newlab))
            return
        # if STATUS_ERROR == 1:
        if end == 'error':
            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.WARN))
            self.OutText.AppendText('\n%s\n' % (count))
            self.ERROR = True
        else:
            self.barProg.SetRange(duration)  # set overall duration range
            self.barProg.SetValue(0)  # reset bar progress
            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.NORM_TEXT))
            self.OutText.AppendText('\n%s : "%s"\n' % (count, fname))

    # ----------------------------------------------------------------------
    def end_proc(self):
        """
        At the end of the process
        """
        if self.ERROR is True:
            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.ERROR_2))
            self.OutText.AppendText(Logging_Console.MSG_taskfailed + '\n')

        elif self.ABORT is True:
            self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.ABORT))
            self.OutText.AppendText(Logging_Console.MSG_interrupted + '\n')

        else:
            if not self.result:
                endmsg = Logging_Console.MSG_completed
                self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.NORM_TEXT))
            else:
                endmsg = Logging_Console.MSG_unfinished
                self.OutText.SetDefaultStyle(wx.TextAttr(Logging_Console.WARN))
            self.parent.statusbar_msg(_('...Finished'), None)
            self.OutText.AppendText(endmsg + '\n')
            self.barProg.SetValue(0)

        self.OutText.AppendText('\n')
        self.button_stop.Enable(False)
        self.button_close.Enable(True)
        self.PARENT_THREAD = None
    # ----------------------------------------------------------------------

    def on_stop(self, event):
        """
        The user change idea and was stop process
        """
        self.PARENT_THREAD.stop()
        self.parent.statusbar_msg(_("wait... I'm aborting"), 'GOLDENROD',
                                  Logging_Console.WHITE)
        self.PARENT_THREAD.join()
        self.parent.statusbar_msg(_("...Interrupted"), None)
        self.ABORT = True

        event.Skip()
    # ----------------------------------------------------------------------

    def on_close(self, event):
        """
        close dialog and retrieve at previusly panel

        """
        if not self.logname:  # there is not process
            self.ckbx_text.Show()
            self.button_stop.Enable(True)
            self.button_close.Enable(False)
            self.parent.panelShown(self.previus)  # retrieve at previusly panel

        if self.PARENT_THREAD is not None:
            if wx.MessageBox(_('There are still processes running.. if you '
                               'want to stop them, use the "Abort" button.\n\n'
                               'Do you want to kill application?'),
                             _('Please confirm'),
                             wx.ICON_QUESTION | wx.YES_NO, self) == wx.NO:
                return

            self.parent.on_Kill()
        # reset all before close
        self.ckbx_text.Show()
        self.button_stop.Enable(True)
        self.button_close.Enable(False)
        self.PARENT_THREAD = None
        self.ABORT = False
        self.ERROR = False
        self.logname = None
        self.result = None
        # self.OutText.Clear()
        # self.labPerc.SetLabel('')
        self.parent.panelShown(self.previus)  # retrieve at previusly panel
        # event.Skip()



# -*- coding: UTF-8 -*-
# Name: ydl_mediainfo.py
# Porpose: show media streams information through youtube-dl.extract_info
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: April.06.2020 *PEP8 compatible*
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################
import wx
import os


class YDL_Mediainfo(wx.MiniFrame):
    """
    Display streams information from youtube-dl data.

    """
    get = wx.GetApp()  # get data from bootstrap

    if get.THEME in ('Breeze-Blues', 'Videomass-Colours'):
        BACKGROUND = '#11303eff'
        FOREGROUND = '#959595'

    elif get.THEME in ('Breeze-Blues', 'Breeze-Dark', 'Videomass-Dark'):
        BACKGROUND = '#1c2027ff'
        FOREGROUND = '#87ceebff'

    else:
        BACKGROUND = '#e6e6faff'
        FOREGROUND = '#191970ff'

    def __init__(self, data, OS):
        """
        NOTE constructor:: with 'None' not depend from videomass.
        With 'parent, -1' if close videomass also close mediainfo window
        """
        self.data = data

        wx.MiniFrame.__init__(self, None, style=wx.CAPTION | wx.CLOSE_BOX |
                              wx.RESIZE_BORDER | wx.SYSTEM_MENU
                              )
        '''constructor'''

        # add panel
        self.panel = wx.Panel(self, wx.ID_ANY, style=wx.TAB_TRAVERSAL |
                              wx.BORDER_THEME)
        # Add widget controls
        self.url_select = wx.ListCtrl(self.panel,
                                      wx.ID_ANY,
                                      style=wx.LC_REPORT |
                                      wx.SUNKEN_BORDER |
                                      wx.LC_SINGLE_SEL
                                      )
        self.textCtrl = wx.TextCtrl(self.panel,
                                    wx.ID_ANY, "",
                                    style=wx.TE_MULTILINE |
                                    wx.TE_READONLY |
                                    wx.TE_RICH2
                                    )
        self.textCtrl.SetBackgroundColour(YDL_Mediainfo.BACKGROUND)
        self.textCtrl.SetDefaultStyle(wx.TextAttr(YDL_Mediainfo.FOREGROUND))
        button_close = wx.Button(self.panel, wx.ID_CLOSE, "")

        # ----------------------Properties----------------------#
        self.SetTitle(_('Statistics viewer'))
        self.SetMinSize((640, 400))
        self.url_select.SetMinSize((640, 200))
        self.url_select.InsertColumn(0, _('TITLE SELECTION'), width=250)
        self.url_select.InsertColumn(1, _('URL'), width=500)
        self.textCtrl.SetMinSize((640, 300))

        if OS == 'Darwin':
            self.url_select.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL,
                                            wx.NORMAL))
            self.textCtrl.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL,
                                          wx.BOLD))
        else:
            self.url_select.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL,
                                            wx.NORMAL))
            self.textCtrl.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL,
                                          wx.NORMAL))
        # ----------------------Layout--------------------------#
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_1.Add(self.url_select, 0, wx.ALL | wx.EXPAND, 5)
        grid_sizer_1 = wx.GridSizer(1, 1, 0, 0)
        sizer_1.Add(grid_sizer_1, 1, wx.EXPAND, 0)

        grid_sizer_1.Add(self.textCtrl, 0, wx.ALL | wx.EXPAND, 5)
        grid_buttons = wx.GridSizer(1, 1, 0, 0)
        grid_buttons.Add(button_close, 1, wx.ALL, 5)
        sizer_1.Add(grid_buttons, flag=wx.ALIGN_RIGHT | wx.RIGHT, border=0)
        self.panel.SetSizer(sizer_1)
        sizer_1.Fit(self)
        self.Layout()

        index = 0
        for url in self.data:
            self.url_select.InsertItem(index, url['title'])
            self.url_select.SetItem(index, 1, url['url'])
            index += 1

        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.url_select)
        self.Bind(wx.EVT_BUTTON, self.on_close, button_close)
        self.Bind(wx.EVT_CLOSE, self.on_close)

        self.url_select.Focus(0)  # make first line the current line selected
        self.url_select.Select(0, on=1)  # default event selection

    # ----------------------Event handler (callback)----------------------#

    def on_select(self, event):
        """
        show data during items selection

        """
        # delete previous append:
        self.textCtrl.Clear()

        index = self.url_select.GetFocusedItem()
        item = self.url_select.GetItemText(index)

        for info in self.data:
            if info['title'] == item:
                text = ("Categories:      {}\n"
                        "License:         {}\n"
                        "Upload Date:     {}\n"
                        "Uploader:        {}\n"
                        "View Count:      {}\n"
                        "Like Count:      {}\n"
                        "Dislike Count:   {}\n"
                        "Average Rating:  {}\n"
                        "ID:              {}\n"
                        "Duration:        {}\n"
                        "Description:     {}\n".format(info['categories'],
                                                       info['license'],
                                                       info['upload_date'],
                                                       info['uploader'],
                                                       info['view'],
                                                       info['like'],
                                                       info['dislike'],
                                                       info['average_rating'],
                                                       info['id'],
                                                       info['duration'],
                                                       info['description'],
                                                       ))
        self.textCtrl.AppendText(text)
    # ------------------------------------------------------------------#

    def on_close(self, event):
        self.Destroy()



# -*- coding: UTF-8 -*-
# Name: ffplay_url_exec.py
# Porpose: playback online media streams with ffplay player
#          using youtube-dl as executable.
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: April.21.2021 *PEP8 compatible*
#########################################################
# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################
import wx
import subprocess
import platform
if not platform.system() == 'Windows':
    import shlex
import os
import sys
from threading import Thread
import time
from pubsub import pub
from videomass3.vdms_io import IO_tools
from videomass3.vdms_io.make_filelog import write_log  # write initial log
if 'youtube_dl' in sys.modules:
    import youtube_dl


def msg_error(msg, title="Videomass"):
    """
    Receive error messages
    """
    wx.MessageBox("%s" % (msg), title, wx.ICON_ERROR)

# ------------------------------------------------------------------------#


class Exec_Download_Stream(Thread):
    """
    Download media stream to the videomass cache directory with
    the same quality defined in the `quality` parameter. The download
    progress continues until an interruption signal is detected.
    The interruption signal is defined by `self.stop_work_thread`
    attribute based from boolean values.
    This Thread use youtube-dl executable (not module) with
    subprocess.Popen class.

    The file name form stored on cache dir. is

            "title_" + "quality" + ".format"

    NOTE see also ffplay_url_lib.py on sources directory.

    """
    get = wx.GetApp()  # get videomass wx.App attribute
    FFMPEG_URL = get.FFMPEG_url
    BINLOCAL = get.execYdl
    EXCEPTION = None
    TMP = get.TMP
    LOGDIR = get.LOGdir
    YDL_PREF = get.YDL_pref
    APPTYPE = get.APP_type

    if platform.system() == 'Windows':
        pyname = 'Scripts', 'youtube-dl.exe'
    else:
        pyname = 'bin', 'youtube-dl'

    if YDL_PREF == 'disabled':
        EXCEPTION = 'error: youtube-dl is disabled, check preferences.'
        EXECYDL = BINLOCAL

    elif APPTYPE == 'pyinstaller':
        if BINLOCAL is not False:
            EXECYDL = BINLOCAL
        else:
            EXCEPTION = 'Not found: %s' % BINLOCAL

    else:
        if 'youtube_dl' in sys.modules:
            # see also inspect: `inspect.getfile(youtube_dl)`
            # or the best: shutil.which('python')
            pypath = youtube_dl.__file__.split('lib')[0]
            EXECYDL = os.path.join(pypath, pyname[0], pyname[1])
        else:
            EXECYDL = False

    if platform.system() == 'Windows':
        if os.path.isfile(EXECYDL):
            LINE_MSG = ('\nRequires MSVCR100.dll\nTo resolve this problem '
                        'install: Microsoft Visual C++ 2010 Redistributable '
                        'Package (x86)')
        else:
            LINE_MSG = ('Unrecognized error')
    else:
        LINE_MSG = ('Unrecognized error')
    # -----------------------------------------------------------------------#

    def __init__(self, url, quality):
        """
        Accept a single url as string and a quality parameter
        in the form required by youtube-dl, e.g one of the
        following quality strings can be valid:

            "worst", "best" "bestvideo+bestaudio" or a "Format Code"
            obtained typing `youtube-dl -F <link>`

        """
        self.logf = os.path.join(Exec_Download_Stream.LOGDIR, 'ffplay.log')
        self.stop_work_thread = False  # process terminate value
        self.url = url
        self.quality = quality

        if (platform.system() == 'Windows' or
                Exec_Download_Stream.APPTYPE == 'appimage'):

            self.ssl = '--no-check-certificate'

        else:
            self.ssl = ''

        write_log('ffplay.log', Exec_Download_Stream.LOGDIR)

        if Exec_Download_Stream.EXCEPTION:
            wx.CallAfter(msg_error, Exec_Download_Stream.EXCEPTION)
            self.logError(Exec_Download_Stream.EXCEPTION)  # append log error
            return

        Thread.__init__(self)
        """initialize Thread class"""
    # ----------------------------------------------------------------------#

    def run(self):
        """
        Starting thread.
        """
        cmd = ('"{0}" {1} --prefer-ffmpeg --newline --ignore-errors -o '
               '"{2}/%(title)s_{3}.%(ext)s" --continue --format {3} '
               '--no-playlist --no-part --ignore-config '
               '--restrict-filenames "{4}" --ffmpeg-location '
               '"{5}"'.format(Exec_Download_Stream.EXECYDL,
                              self.ssl,
                              Exec_Download_Stream.TMP,
                              self.quality,
                              self.url,
                              Exec_Download_Stream.FFMPEG_URL,
                              ))
        self.logWrite(cmd)  # append log cmd
        if not platform.system() == 'Windows':
            cmd = shlex.split(cmd)
            info = None
        else:
            # Hide subprocess window on MS Windows
            info = subprocess.STARTUPINFO()
            info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        try:
            with subprocess.Popen(cmd,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.STDOUT,
                                  bufsize=1,
                                  universal_newlines=True,
                                  startupinfo=info,) as p:

                for line in p.stdout:
                    if not self.stop_work_thread:
                        if '[download]' in line:  # ...in processing
                            if 'Destination' in line:
                                pub.sendMessage("START_FFPLAY_EVT",
                                                output=line.split()[2]
                                                )
                            elif 'has already been downloaded' in line:
                                pub.sendMessage("START_FFPLAY_EVT",
                                                output=line.split()[1]
                                                )
                    else:  # self.stop_work_thread:
                        p.terminate()
                        break

                if p.wait():  # error at end process
                    if 'line' not in locals():
                        line = Exec_Download_Stream.LINE_MSG
                    if '[download]' in line:
                        return
                    wx.CallAfter(msg_error,
                                 line,
                                 'Videomass: Error %s' % p.wait()
                                 )
                    self.logError(line)  # append log error
                    return

        except OSError as err:
            wx.CallAfter(msg_error, err, 'Videomass: OSError')
            self.logError(err)  # append log error
            return
    # --------------------------------------------------------------------#

    def stop(self):
        """
        Sets the stop work thread to terminate the process
        """
        self.stop_work_thread = True
    # ----------------------------------------------------------------#

    def logWrite(self, cmd):
        """
        write ffplay command log
        """
        with open(self.logf, "a", encoding='utf8') as log:
            log.write("%s\n" % (cmd))
    # ----------------------------------------------------------------#

    def logError(self, error):
        """
        write ffplay errors
        """
        with open(self.logf, "a", encoding='utf8') as logerr:
            logerr.write("\n[FFMPEG] FFplay "
                         "ERRORS:\n%s\n" % (error))
# ------------------------------------------------------------------------#


class Exec_Streaming(object):
    """
    Handling Threads to download and playback media streams via
    youtube-dl and ffplay executables.

    DOWNLOAD class variable makes the object's attributes available
    even outside of class, see `stop_download()`
    FIXME put RICEIVER LISTENER function as methods of this class. For
          now does not work (???)

    """
    DOWNLOAD = None  # set instance thread
    TIMESTAMP = None
    AUTOEXIT = None
    # ---------------------------------------------------------------#

    def __init__(self, timestamp, autoexit, url=None, quality=None):
        """
        - Topic "START_FFPLAY_EVT" subscribes the start playing
          running ffplay at a certain time.
        - Topic "STOP_DOWNLOAD_EVT" subscribes a stop download listener
          which call the stop() method of `Download_Stream` class to
          stop the download when ffplay has finished or been closed by
          the user.

        """
        pub.subscribe(stop_download_listener, "STOP_DOWNLOAD_EVT")
        pub.subscribe(start_palying_listener, "START_FFPLAY_EVT")

        Exec_Streaming.DOWNLOAD = Exec_Download_Stream(url, quality)
        Exec_Streaming.TIMESTAMP = timestamp
        Exec_Streaming.AUTOEXIT = autoexit

        self.start_download()
    # ----------------------------------------------------------------#

    def start_download(self):
        """
        call Exec_Download_Stream(Thread) to run() method

        """
        if not Exec_Download_Stream.EXCEPTION:
            Exec_Streaming.DOWNLOAD.start()
        return

# --------- RECEIVER LISTENERS


def stop_download_listener(filename):
    """
    Receive message from ffplay_file.File_Play class
    for handle interruption

    """
    Exec_Streaming.DOWNLOAD.stop()
    Exec_Streaming.DOWNLOAD.join()

    return


def start_palying_listener(output):
    """
    Riceive message from Exec_Download_Stream class to start
    ffplay in at a given time.

    """
    IO_tools.stream_play(output, '', Exec_Streaming.TIMESTAMP,
                         Exec_Streaming.AUTOEXIT)
    return



# -*- coding: UTF-8 -*-
# Name: configurator.py
# Porpose: Set Videomass configuration and appearance on startup
# Compatibility: Python3
# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: April.21.2021 *PEP8 compatible*
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################
import os
import sys
import shutil
from videomass3.vdms_utils.utils import copydir_recursively
from videomass3.vdms_utils.utils import copy_restore
import platform


class Data_Source(object):
    """
    Data_Source class determines the Videomass's configuration
    according to the Operating System and define the environment
    paths based on the program execution and/or where it's
    installed.
    """
    USER_NAME = os.path.expanduser('~')
    OS = platform.system()

    # Establish the conventional paths based on OS
    if OS == 'Windows':
        DIRPATH = "\\AppData\\Roaming\\videomass\\videomass.conf"
        FILE_CONF = os.path.join(USER_NAME + DIRPATH)
        DIR_CONF = os.path.join(USER_NAME + "\\AppData\\Roaming\\videomass")
        LOG_DIR = os.path.join(DIR_CONF, 'log')  # logs
        CACHE_DIR = os.path.join(DIR_CONF, 'cache')  # updates executable

    elif OS == "Darwin":
        DIRPATH = "Library/Application Support/videomass/videomass.conf"
        FILE_CONF = os.path.join(USER_NAME, DIRPATH)
        DIR_CONF = os.path.join(USER_NAME, os.path.dirname(DIRPATH))
        LOG_DIR = os.path.join(USER_NAME, "Library/Logs/videomass")
        CACHE_DIR = os.path.join(USER_NAME, "Library/Caches/videomass")

    else:  # Linux, FreeBsd, etc.
        DIRPATH = ".config/videomass/videomass.conf"
        FILE_CONF = os.path.join(USER_NAME, DIRPATH)
        DIR_CONF = os.path.join(USER_NAME, ".config/videomass")
        LOG_DIR = os.path.join(USER_NAME, ".local/share/videomass/log")
        CACHE_DIR = os.path.join(USER_NAME, ".cache/videomass")
    # -------------------------------------------------------------------

    def __init__(self):
        """
        Given the paths defined by `data_location` (a configuration
        folder for recovery > `self.SRCpath`, a set of icons >
        `self.icodir` and a folder for the locale > `self.localepath`),
        it performs the initialization described in Data_Source.
        """
        self.apptype = None  # appimage, pyinstaller on None

        if getattr(sys, 'frozen', False) and hasattr(sys, '_MEIPASS'):
            print('getattr frozen, hasattr _MEIPASS')
            self.apptype = 'pyinstaller'
            frozen, meipass = True, True
            path = getattr(sys, '_MEIPASS',  os.path.abspath(__file__))
            data_location = path

        else:
            frozen, meipass = False, False
            path = os.path.realpath(os.path.abspath(__file__))
            data_location = os.path.dirname(os.path.dirname(path))

        self.WORKdir = os.path.dirname(os.path.dirname(os.path.dirname(path)))
        self.localepath = os.path.join(data_location, 'locale')
        self.SRCpath = os.path.join(data_location, 'share')
        self.icodir = os.path.join(data_location, 'art', 'icons')
        self.FFmpeglocaldir = os.path.join(data_location, 'FFMPEG')
        launcher = os.path.isfile('%s/launcher' % self.WORKdir)

        if frozen and meipass or launcher:
            print('frozen=%s meipass=%s launcher=%s' % (frozen,
                                                        meipass,
                                                        launcher
                                                        ))
            self.videomass_icon = "%s/videomass.png" % self.icodir

        else:
            if Data_Source.OS == 'Windows':  # Installed with pip command
                print('Win32 executable=%s' % sys.executable)
                # HACK check this
                dirname = os.path.dirname(sys.executable)
                pythonpath = os.path.join(dirname, 'Script', 'videomass')
                # self.icodir = dirname + '\\share\\videomass\\icons'
                self.videomass_icon = self.icodir + "\\videomass.png"

            elif ('/tmp/.mount_' in sys.executable or os.path.exists(
                  os.path.dirname(os.path.dirname(os.path.dirname(
                   sys.argv[0]))) + '/AppRun')):
                # embedded on python appimage
                print('Embedded on python appimage')
                self.apptype = 'appimage'
                userbase = os.path.dirname(os.path.dirname(sys.argv[0]))
                pixmaps = '/share/pixmaps/videomass.png'
                self.videomass_icon = os.path.join(userbase + pixmaps)

            elif 'usr/bin/videomass' in sys.argv[0]:
                print('Embedded on python appimage externally')
                self.apptype = 'appimage'
                userbase = os.path.dirname(os.path.dirname(sys.argv[0]))
                pixmaps = '/share/pixmaps/videomass.png'
                self.videomass_icon = os.path.join(userbase + pixmaps)

            else:
                binarypath = shutil.which('videomass')

                if binarypath == '/usr/local/bin/videomass':
                    print('executable=%s' % binarypath)
                    # pip as super user, usually Linux, MacOs, Unix
                    share = '/usr/local/share/pixmaps'
                    self.videomass_icon = share + '/videomass.png'

                elif binarypath == '/usr/bin/videomass':
                    print('executable=%s' % binarypath)
                    # installed via apt, rpm, etc, usually Linux
                    share = '/usr/share/pixmaps'
                    self.videomass_icon = share + "/videomass.png"

                else:
                    print('executable=%s' % binarypath)
                    # pip as normal user, usually Linux, MacOs, Unix
                    userbase = os.path.dirname(os.path.dirname(binarypath))
                    pixmaps = '/share/pixmaps/videomass.png'
                    self.videomass_icon = os.path.join(userbase + pixmaps)
    # ---------------------------------------------------------------------

    def parsing_fileconf(self):
        """
        Make parsing of the configuration file and return
        object list with the current program settings data.
        """
        with open(Data_Source.FILE_CONF, 'r', encoding='utf8') as f:
            fconf = f.readlines()
        lst = [line.strip() for line in fconf if not line.startswith('#')]
        dataconf = [x for x in lst if x]  # list without empties values
        if not dataconf:
            return
        return dataconf
    # --------------------------------------------------------------------

    def get_fileconf(self):
        """
        Get videomass configuration data from videomass.conf .

        This method performs the following main steps:

            1) Checks user videomass configuration dir; if not exists try to
               restore from self.SRCpath
            2) Read the videomass.conf file; if not exists try to restore
               from self.SRCpath
            3) Checks presets folder; if not exists try to restore from
               self.SRCpath

        Note that when `copyerr` is not False, it causes a fatal error on
        videomass bootstrap.
        """
        copyerr = False
        existfileconf = True  # True > found, False > not found

        if os.path.exists(Data_Source.DIR_CONF):  # if ~/.conf/videomass dir
            if os.path.isfile(Data_Source.FILE_CONF):
                userconf = self.parsing_fileconf()  # fileconf data
                if not userconf:
                    existfileconf = False
                if float(userconf[0]) != 3.1:
                    existfileconf = False
            else:
                existfileconf = False

            if not existfileconf:  # try to restore only videomass.conf
                fcopy = copy_restore('%s/videomass.conf' % self.SRCpath,
                                     Data_Source.FILE_CONF)
                if fcopy:
                    copyerr, userconf = fcopy, None
                else:
                    userconf = self.parsing_fileconf()  # read again file conf

            if not os.path.exists(os.path.join(Data_Source.DIR_CONF,
                                               "presets")):
                # try to restoring presets directory on videomass dir
                drest = copydir_recursively(os.path.join(self.SRCpath,
                                                         "presets"),
                                            Data_Source.DIR_CONF)
                if drest:
                    copyerr, userconf = drest, None

        else:  # try to restore entire configuration directory
            dconf = copydir_recursively(self.SRCpath,
                                        os.path.dirname(Data_Source.DIR_CONF),
                                        "videomass")
            userconf = self.parsing_fileconf()  # read again file conf
            if dconf:
                copyerr, userconf = dconf, None

        return (Data_Source.OS,
                self.SRCpath,
                copyerr,
                None,  # unused
                userconf,
                self.localepath,
                Data_Source.FILE_CONF,
                self.WORKdir,
                Data_Source.DIR_CONF,
                Data_Source.LOG_DIR,
                Data_Source.CACHE_DIR,
                self.FFmpeglocaldir,
                self.apptype,
                )
    # --------------------------------------------------------------------

    def icons_set(self, iconset):
        """
        Assignment paths at the used icons in according to
        configuration file.

        """
        if 'wx.svg' in sys.modules:
            ext = 'svg'
        else:
            ext = 'png'
        #  Videomass icons for light themes
        if iconset == 'Videomass-Light':
            x48 = '%s/Sign_Icons/48x48_light' % self.icodir
            x16 = '%s/Videomass-Light/16x16' % self.icodir
            x22 = '%s/Videomass-Light/24x24' % self.icodir
        #  Videomass icons for dark themes
        elif iconset == 'Videomass-Dark':
            x48 = '%s/Sign_Icons/48x48_dark' % self.icodir
            x16 = '%s/Videomass-Dark/16x16' % self.icodir
            x22 = '%s/Videomass-Dark/24x24' % self.icodir
        #  Videomass icons for all themes
        elif iconset == 'Videomass-Colours':
            x48 = '%s/Sign_Icons/48x48' % self.icodir
            x16 = '%s/Videomass-Colours/16x16' % self.icodir
            x22 = '%s/Videomass-Colours/24x24' % self.icodir
        # Breeze for light themes
        elif iconset == 'Breeze':  # default
            x48 = '%s/Sign_Icons/48x48_light' % self.icodir
            x16 = '%s/Breeze/16x16' % self.icodir
            x22 = '%s/Breeze/22x22' % self.icodir
        # breeze for dark themes
        elif iconset == 'Breeze-Dark':
            x48 = '%s/Sign_Icons/48x48_dark' % self.icodir
            x16 = '%s/Breeze-Dark/16x16' % self.icodir
            x22 = '%s/Breeze-Dark/22x22' % self.icodir
        # breeze custom icons colorized for all themes
        elif iconset == 'Breeze-Blues':
            x48 = '%s/Sign_Icons/48x48' % self.icodir
            x16 = '%s/Breeze-Blues/16x16' % self.icodir
            x22 = '%s/Breeze-Blues/22x22' % self.icodir

        # choose topic icons 48x48:
        icon_switchvideomass = '%s/icon_videoconversions.%s' % (x48, ext)
        icon_youtube = '%s/icon_youtube.%s' % (x48, ext)
        icon_prst_mng = '%s/icon_prst_mng.%s' % (x48, ext)
        icon_concat = '%s/icon_concat.%s' % (x48, ext)
        # toolbar icons 22x22:
        icon_process = '%s/convert.%s' % (x22, ext)
        icon_toolback = '%s/go-previous.%s' % (x22, ext)
        icon_toolforward = '%s/go-next.%s' % (x22, ext)
        icon_ydl = '%s/download.%s' % (x22, ext)
        icn_infosource = '%s/properties.%s' % (x22, ext)
        icn_saveprf = '%s/profile-append.%s' % (x22, ext)
        icn_viewstatistics = '%s/statistics.%s' % (x22, ext)
        # button icons 16x16:
        icn_playback = '%s/playback.%s' % (x16, ext)
        icn_preview = '%s/preview.%s' % (x16, ext)
        icn_resetfilters = '%s/edit-clear.%s' % (x16, ext)
        ic_resize = '%s/transform-scale.%s' % (x16, ext)
        ic_crop = '%s/transform-crop.%s' % (x16, ext)
        ic_rotate = '%s/transform-rotate.%s' % (x16, ext)
        ic_deinterlace = '%s/deinterlace.%s' % (x16, ext)
        ic_denoiser = '%s/denoise.%s' % (x16, ext)
        ic_analyzes = '%s/statistics.%s' % (x16, ext)
        ic_settings = '%s/configure.%s' % (x16, ext)
        ic_peaklevel = '%s/player-volume.%s' % (x16, ext)
        icn_newprf = '%s/newprf.%s' % (x16, ext)
        icn_delprf = '%s/delprf.%s' % (x16, ext)
        icn_editprf = '%s/editprf.%s' % (x16, ext)
        icn_vidstab = '%s/stabilizer.%s' % (x16, ext)

        return [os.path.join(norm) for norm in [self.videomass_icon,  # 0
                                                icon_switchvideomass,  # 1
                                                icon_process,  # 2
                                                icn_infosource,  # 3
                                                icn_playback,  # 4
                                                icon_concat,  # 5
                                                icn_preview,  # 6
                                                icn_resetfilters,   # 7
                                                icn_saveprf,  # 8
                                                ic_resize,  # 9
                                                ic_crop,  # 10
                                                ic_rotate,  # 11
                                                ic_deinterlace,  # 12
                                                ic_denoiser,  # 13
                                                ic_analyzes,  # 14
                                                ic_settings,  # 15
                                                ic_peaklevel,  # 16
                                                icon_youtube,  # 17
                                                icon_prst_mng,  # 18
                                                icn_newprf,  # 19
                                                icn_delprf,  # 20
                                                icn_editprf,  # 21
                                                icon_toolback,  # 22
                                                icon_toolforward,  # 23
                                                icon_ydl,  # 24
                                                icn_viewstatistics,  # 25
                                                icn_vidstab,  # 26
                                                ]]



# -*- coding: UTF-8 -*-
# Name: one_pass.py
# Porpose: FFmpeg long processing task on one pass conversion
# Compatibility: Python3, wxPython4 Phoenix
# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: April.06.2020 *PEP8 compatible*
#########################################################
# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################
import wx
import subprocess
import platform
if not platform.system() == 'Windows':
    import shlex
import itertools
import os
from threading import Thread
import time
from pubsub import pub


def logWrite(cmd, sterr, logname, logdir):
    """
    writes ffmpeg commands and status error during threads below
    """
    if sterr:
        apnd = "...%s\n\n" % (sterr)
    else:
        apnd = "%s\n\n" % (cmd)

    with open(os.path.join(logdir, logname), "a", encoding='utf8') as log:
        log.write(apnd)

# ------------------------------ THREADS -------------------------------#


"""
NOTE MS Windows:

subprocess.STARTUPINFO()

https://stackoverflow.com/questions/1813872/running-
a-process-in-pythonw-with-popen-without-a-console?lq=1>

NOTE capturing output in real-time (Windows, Unix):

https://stackoverflow.com/questions/1388753/how-to-get-output-
from-subprocess-popen-proc-stdout-readline-blocks-no-dat?rq=1

"""


class OnePass(Thread):
    """
    This class represents a separate thread for running processes,
    which need to read the stdout/stderr in real time.

    """
    # get videomass wx.App attribute
    get = wx.GetApp()
    LOGDIR = get.LOGdir
    FFMPEG_URL = get.FFMPEG_url
    FFMPEG_LOGLEV = get.FFMPEG_loglev
    FF_THREADS = get.FFthreads
    SUFFIX = '' if get.FILEsuffix == 'none' else get.FILEsuffix
    NOT_EXIST_MSG = _("Is 'ffmpeg' installed on your system?")
    # ---------------------------------------------------------------

    def __init__(self, varargs, duration, logname, timeseq):
        """
        Some attribute can be empty, this depend from conversion type.
        If the format/container is not changed on a conversion, the
        'extoutput' attribute will have an empty value.
        The 'volume' attribute may also have an empty value, but it will
        have no influence on the type of conversion.
        """
        self.stop_work_thread = False  # process terminate
        self.filelist = varargs[1]  # list of files (items)
        self.command = varargs[4]  # comand set on single pass
        self.outputdir = varargs[3]  # output path
        self.extoutput = varargs[2]  # format (extension)
        self.duration = duration  # duration list
        self.volume = varargs[7]  # (lista norm.)se non richiesto rimane None
        self.count = 0  # count first for loop
        self.countmax = len(varargs[1])  # length file list
        self.logname = logname  # title name of file log
        self.time_seq = timeseq  # a time segment

        Thread.__init__(self)
        """initialize"""
        self.start()  # start the thread (va in self.run())

    def run(self):
        """
        Subprocess initialize thread.
        """
        for (files,
             folders,
             volume,
             duration) in itertools.zip_longest(self.filelist,
                                                self.outputdir,
                                                self.volume,
                                                self.duration,
                                                fillvalue='',
                                                ):
            basename = os.path.basename(files)  # nome file senza path
            filename = os.path.splitext(basename)[0]  # nome senza estensione
            source_ext = os.path.splitext(basename)[1].split('.')[1]  # ext
            outext = source_ext if not self.extoutput else self.extoutput

            cmd = ('"%s" %s %s -i "%s" %s %s %s '
                   '-y "%s/%s%s.%s"' % (OnePass.FFMPEG_URL,
                                        self.time_seq,
                                        OnePass.FFMPEG_LOGLEV,
                                        files,
                                        self.command,
                                        volume,
                                        OnePass.FF_THREADS,
                                        folders,
                                        filename,
                                        OnePass.SUFFIX,
                                        outext,
                                        ))
            self.count += 1
            count = 'File %s/%s' % (self.count, self.countmax,)
            com = "%s\n%s" % (count, cmd)
            wx.CallAfter(pub.sendMessage,
                         "COUNT_EVT",
                         count=count,
                         duration=duration,
                         fname=files,
                         end='',
                         )
            logWrite(com,
                     '',
                     self.logname,
                     OnePass.LOGDIR,
                     )  # write n/n + command only

            if not platform.system() == 'Windows':
                cmd = shlex.split(cmd)
                info = None
            else:  # Hide subprocess window on MS Windows
                info = subprocess.STARTUPINFO()
                info.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            try:
                with subprocess.Popen(cmd,
                                      stderr=subprocess.PIPE,
                                      bufsize=1,
                                      universal_newlines=True,
                                      startupinfo=info,) as p:
                    for line in p.stderr:
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output=line,
                                     duration=duration,
                                     status=0,
                                     )
                        if self.stop_work_thread:
                            p.terminate()
                            break  # break second 'for' loop

                    if p.wait():  # error
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output=line,
                                     duration=duration,
                                     status=p.wait(),
                                     )
                        logWrite('',
                                 "Exit status: %s" % p.wait(),
                                 self.logname,
                                 OnePass.LOGDIR,
                                 )  # append exit error number
                    else:  # ok
                        wx.CallAfter(pub.sendMessage,
                                     "COUNT_EVT",
                                     count='',
                                     duration='',
                                     fname='',
                                     end='ok'
                                     )
            except (OSError, FileNotFoundError) as err:
                e = "%s\n  %s" % (err, OnePass.NOT_EXIST_MSG)
                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count=e,
                             duration=0,
                             fname=files,
                             end='error',
                             )
                break

            if self.stop_work_thread:
                p.terminate()
                break  # break second 'for' loop

        time.sleep(.5)
        wx.CallAfter(pub.sendMessage, "END_EVT")
    # --------------------------------------------------------------------#

    def stop(self):
        """
        Sets the stop work thread to terminate the process
        """
        self.stop_work_thread = True



# -*- coding: UTF-8 -*-
# Name: mediainfo.py
# Porpose: show media streams information through ffprobe
# Compatibility: Python3, wxPython Phoenix
# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: Feb.03.2021 *PEP8 compatible*
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################
import wx
import os
import webbrowser


class Mediainfo(wx.MiniFrame):
    """
    Display streams information from ffprobe json data.
    """
    get = wx.GetApp()  # get data from bootstrap

    if get.THEME in ('Breeze-Blues', 'Videomass-Colours'):
        BACKGROUND = '#11303eff'
        FOREGROUND = '#959595'

    elif get.THEME in ('Breeze-Blues', 'Breeze-Dark', 'Videomass-Dark'):
        BACKGROUND = '#1c2027ff'
        FOREGROUND = '#87ceebff'

    else:
        BACKGROUND = '#e6e6faff'
        FOREGROUND = '#191970ff'

    def __init__(self, data, OS):
        """
        self.data: list object containing ffprobe data. See FFprobe
        class on vdms_threads/ffprobe_parser.py and probeInfo on
        vdms_io/IO_tools.py
        """
        self.data = data
        msg = _('Display of selected items in text format')
        msg1 = _('Select items to view them in text format')

        wx.MiniFrame.__init__(self, None, style=wx.CAPTION | wx.CLOSE_BOX |
                              wx.RESIZE_BORDER | wx.SYSTEM_MENU
                              )
        '''NOTE constructor: with 'None' not depend from videomass.
        With 'parent, -1' if close videomass also close mediainfo window'''

        # add panel
        self.panel = wx.Panel(self, wx.ID_ANY, style=wx.TAB_TRAVERSAL |
                              wx.BORDER_THEME)
        self.sizerBase = wx.BoxSizer(wx.VERTICAL)
        self.file_select = wx.ListCtrl(self.panel,
                                       wx.ID_ANY,
                                       style=wx.LC_REPORT |
                                       wx.SUNKEN_BORDER |
                                       wx.LC_SINGLE_SEL
                                       )
        self.sizerBase.Add(self.file_select, 0, wx.ALL | wx.EXPAND, 5)

        notebook = wx.Notebook(self.panel, wx.ID_ANY)
        grid_notebook = wx.GridSizer(1, 1, 0, 0)
        grid_notebook.Add(notebook, 1, wx.ALL | wx.EXPAND, 5)
        self.sizerBase.Add(grid_notebook, 1, wx.EXPAND, 0)
        #  tab format
        nb_panel_1 = wx.Panel(notebook, wx.ID_ANY)
        self.format_ctrl = wx.ListCtrl(nb_panel_1,
                                       wx.ID_ANY,
                                       style=wx.LC_REPORT |
                                       wx.SUNKEN_BORDER |
                                       wx.LC_SINGLE_SEL
                                       )
        self.format_stxt = wx.StaticText(nb_panel_1, wx.ID_ANY, msg)
        self.format_tags = wx.TextCtrl(nb_panel_1,
                                       wx.ID_ANY, "",
                                       style=wx.TE_MULTILINE |
                                       wx.TE_READONLY |
                                       wx.TE_RICH2
                                       )
        sizer_tab1 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab1.Add(self.format_ctrl, 1, wx.ALL | wx.EXPAND, 5)
        sizer_tab1.Add(self.format_stxt, 0, wx.ALL, 5)
        sizer_tab1.Add(self.format_tags, 0, wx.ALL | wx.EXPAND, 5)
        nb_panel_1.SetSizer(sizer_tab1)
        notebook.AddPage(nb_panel_1, (_("Data Format")))
        #  tab video
        nb_panel_2 = wx.Panel(notebook, wx.ID_ANY)

        self.video_ctrl = wx.ListCtrl(nb_panel_2,
                                      wx.ID_ANY,
                                      style=wx.LC_REPORT |
                                      wx.SUNKEN_BORDER |
                                      wx.LC_SINGLE_SEL
                                      )
        self.video_stxt = wx.StaticText(nb_panel_2, wx.ID_ANY, msg)
        self.video_tags = wx.TextCtrl(nb_panel_2,
                                      wx.ID_ANY, "",
                                      style=wx.TE_MULTILINE |
                                      wx.TE_READONLY |
                                      wx.TE_RICH2
                                      )
        sizer_tab2 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab2.Add(self.video_ctrl, 1, wx.ALL | wx.EXPAND, 5)
        sizer_tab2.Add(self.video_stxt, 0, wx.ALL, 5)
        sizer_tab2.Add(self.video_tags, 0, wx.ALL | wx.EXPAND, 5)
        nb_panel_2.SetSizer(sizer_tab2)
        notebook.AddPage(nb_panel_2, (_("Video Stream")))
        #  tab audio
        nb_panel_3 = wx.Panel(notebook, wx.ID_ANY)
        self.audio_ctrl = wx.ListCtrl(nb_panel_3,
                                      wx.ID_ANY,
                                      style=wx.LC_REPORT |
                                      wx.SUNKEN_BORDER |
                                      wx.LC_SINGLE_SEL
                                      )
        self.audio_stxt = wx.StaticText(nb_panel_3, wx.ID_ANY, msg)
        self.audio_tags = wx.TextCtrl(nb_panel_3,
                                      wx.ID_ANY, "",
                                      style=wx.TE_MULTILINE |
                                      wx.TE_READONLY |
                                      wx.TE_RICH2
                                      )
        sizer_tab3 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab3.Add(self.audio_ctrl, 1, wx.ALL | wx.EXPAND, 5)
        sizer_tab3.Add(self.audio_stxt, 0, wx.ALL, 5)
        sizer_tab3.Add(self.audio_tags, 0, wx.ALL | wx.EXPAND, 5)
        nb_panel_3.SetSizer(sizer_tab3)
        notebook.AddPage(nb_panel_3, (_("Audio Streams")))
        #  tab subtitle
        nb_panel_4 = wx.Panel(notebook, wx.ID_ANY)
        self.subtitle_ctrl = wx.ListCtrl(nb_panel_4,
                                         wx.ID_ANY,
                                         style=wx.LC_REPORT |
                                         wx.SUNKEN_BORDER |
                                         wx.LC_SINGLE_SEL
                                         )
        self.sub_stxt = wx.StaticText(nb_panel_4, wx.ID_ANY, msg)
        self.sub_tags = wx.TextCtrl(nb_panel_4,
                                    wx.ID_ANY, "",
                                    style=wx.TE_MULTILINE |
                                    wx.TE_READONLY |
                                    wx.TE_RICH2
                                    )
        sizer_tab4 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab4.Add(self.subtitle_ctrl, 1, wx.ALL | wx.EXPAND, 5)
        sizer_tab4.Add(self.sub_stxt, 0, wx.ALL, 5)
        sizer_tab4.Add(self.sub_tags, 0, wx.ALL | wx.EXPAND, 5)
        nb_panel_4.SetSizer(sizer_tab4)
        notebook.AddPage(nb_panel_4, (_("Subtitle Streams")))
        #  bottom
        button_close = wx.Button(self.panel, wx.ID_CLOSE, "")
        self.button_view = wx.ToggleButton(self.panel, wx.ID_ANY,
                                           _("Enable text format"))
        grid_btns = wx.GridSizer(1, 2, 0, 0)
        grid_btns.Add(self.button_view, 0, wx.ALL | wx.EXPAND, 5)
        grid_btns.Add(button_close, 0, wx.ALL | wx.EXPAND, 5)
        self.sizerBase.Add(grid_btns, flag=wx.ALIGN_RIGHT | wx.RIGHT, border=0)

        # ----------------------Properties----------------------#
        self.format_tags.Hide(), self.format_stxt.Hide()
        self.video_tags.Hide(), self.video_stxt.Hide()
        self.audio_tags.Hide(), self.audio_stxt.Hide()
        self.sub_tags.Hide(), self.sub_stxt.Hide()

        self.file_select.InsertColumn(0, _('FILE SELECTION'), width=700)

        self.format_ctrl.InsertColumn(0, _('References'), width=200)
        self.format_ctrl.InsertColumn(1, _('Parameters'), width=500)

        self.video_ctrl.InsertColumn(0, _('References'), width=200)
        self.video_ctrl.InsertColumn(1, _('Parameters'), width=500)

        self.audio_ctrl.InsertColumn(0, _('References'), width=200)
        self.audio_ctrl.InsertColumn(1, _('Parameters'), width=500)

        self.subtitle_ctrl.InsertColumn(0, _('References'), width=200)
        self.subtitle_ctrl.InsertColumn(1, _('Parameters'), width=500)

        self.format_tags.SetBackgroundColour(Mediainfo.BACKGROUND)
        self.format_tags.SetDefaultStyle(wx.TextAttr(Mediainfo.FOREGROUND))
        self.video_tags.SetBackgroundColour(Mediainfo.BACKGROUND)
        self.video_tags.SetDefaultStyle(wx.TextAttr(Mediainfo.FOREGROUND))
        self.audio_tags.SetBackgroundColour(Mediainfo.BACKGROUND)
        self.audio_tags.SetDefaultStyle(wx.TextAttr(Mediainfo.FOREGROUND))
        self.sub_tags.SetBackgroundColour(Mediainfo.BACKGROUND)
        self.sub_tags.SetDefaultStyle(wx.TextAttr(Mediainfo.FOREGROUND))

        self.format_ctrl.SetToolTip(msg1)
        self.video_ctrl.SetToolTip(msg1)
        self.audio_ctrl.SetToolTip(msg1)
        self.subtitle_ctrl.SetToolTip(msg1)

        # set layout
        self.SetTitle(_('Multimedia streams analyzer'))
        self.SetMinSize((700, 500))
        self.file_select.SetMinSize((-1, 150))
        # self.format_ctrl.SetMinSize((-1, 300))
        # self.format_tags.SetMinSize((-1, 100))

        self.format_tags.SetMinSize((-1, 100))
        self.video_tags.SetMinSize((-1, 100))
        self.audio_tags.SetMinSize((-1, 100))
        self.sub_tags.SetMinSize((-1, 100))

        self.panel.SetSizer(self.sizerBase)
        # self.sizerBase.Fit(self)
        self.Fit()
        self.Layout()
        self.CentreOnScreen()

        flist = [x['format']['filename'] for x in self.data
                 if x['format']['filename']]

        index = 0
        for f in flist:
            self.file_select.InsertItem(index, f)
            index += 1

        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.file_select)

        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_format, self.format_ctrl)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_video, self.video_ctrl)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_audio, self.audio_ctrl)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_sub, self.subtitle_ctrl)

        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.format_deselect,
                  self.format_ctrl)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.video_deselect,
                  self.video_ctrl)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.audio_deselect,
                  self.audio_ctrl)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.sub_deselect,
                  self.subtitle_ctrl)

        self.Bind(wx.EVT_TOGGLEBUTTON, self.on_view, self.button_view)
        self.Bind(wx.EVT_BUTTON, self.on_close, button_close)
        self.Bind(wx.EVT_CLOSE, self.on_close)  # controlla la chiusura (x)

        self.file_select.Focus(0)  # make the line the current line
        self.file_select.Select(0, on=1)  # default event selection

    # ----------------------Event handler (callbacks)----------------------#

    def on_view(self, event):
        """
        Show or hide text boxes
        """
        if self.button_view.GetValue():
            self.format_tags.Show(), self.format_stxt.Show()
            self.video_tags.Show(), self.video_stxt.Show()
            self.audio_tags.Show(), self.audio_stxt.Show()
            self.sub_tags.Show(), self.sub_stxt.Show()
        else:
            self.format_tags.Hide(), self.format_stxt.Hide()
            self.video_tags.Hide(), self.video_stxt.Hide()
            self.audio_tags.Hide(), self.audio_stxt.Hide()
            self.sub_tags.Hide(), self.sub_stxt.Hide()

        self.format_tags.SetMinSize((-1, 100))
        self.video_tags.SetMinSize((-1, 100))
        self.audio_tags.SetMinSize((-1, 100))
        self.sub_tags.SetMinSize((-1, 100))
        # self.panel.SetSizer(self.sizerBase)
        self.sizerBase.Fit(self)
        self.Layout()
    # ------------------------------------------------------------------#

    def format_deselect(self, event):
        """
        Delete the text previously added on corresponding text boxes
        """
        self.format_tags.Clear()
    # ------------------------------------------------------------------#

    def video_deselect(self, event):
        """
        Delete the text previously added on corresponding text boxes
        """
        self.video_tags.Clear()
    # ------------------------------------------------------------------#

    def audio_deselect(self, event):
        """
        Delete the text previously added on corresponding text boxes
        """
        self.audio_tags.Clear()
    # ------------------------------------------------------------------#

    def sub_deselect(self, event):
        """
        Delete the text previously added on corresponding text boxes
        """
        self.sub_tags.Clear()
    # ------------------------------------------------------------------#

    def on_format(self, event):
        """
        Shows the data of the selected item in the 'format' text box
        """
        if not self.button_view.GetValue():
            self.button_view.SetValue(True)
            self.on_view(self)
        item = self.format_ctrl.GetFocusedItem()
        col0 = self.format_ctrl.GetItemText(item, col=0)
        col1 = self.format_ctrl.GetItemText(item, col=1)
        self.format_tags.Clear()
        if col0 == 'tags':
            tags = eval(col1)
            for k, v in tags.items():
                self.format_tags.AppendText('%s: %s\n' % (k, v))
        elif col0 == 'disposition':
            dispos = eval(col1)
            for k, v in dispos.items():
                self.format_tags.AppendText('%s: %s\n' % (k, v))

        else:
            self.format_tags.write('%s: %s' % (col0, col1))
    # ------------------------------------------------------------------#

    def on_video(self, event):
        """
        Shows the data of the selected item in the video text box
        """
        if not self.button_view.GetValue():
            self.button_view.SetValue(True)
            self.on_view(self)
        item = self.video_ctrl.GetFocusedItem()
        col0 = self.video_ctrl.GetItemText(item, col=0)
        col1 = self.video_ctrl.GetItemText(item, col=1)
        self.video_tags.Clear()
        if col0 == 'tags':
            tags = eval(col1)
            for k, v in tags.items():
                self.video_tags.AppendText('%s: %s\n' % (k, v))
        elif col0 == 'disposition':
            dispos = eval(col1)
            for k, v in dispos.items():
                self.video_tags.AppendText('%s: %s\n' % (k, v))
        else:
            self.video_tags.AppendText('%s: %s\n' % (col0, col1))
    # ------------------------------------------------------------------#

    def on_audio(self, event):
        """
        Shows the data of the selected item in the audio text box
        """
        if not self.button_view.GetValue():
            self.button_view.SetValue(True)
            self.on_view(self)
        item = self.audio_ctrl.GetFocusedItem()
        col0 = self.audio_ctrl.GetItemText(item, col=0)
        col1 = self.audio_ctrl.GetItemText(item, col=1)
        self.audio_tags.Clear()
        if col0 == 'tags':
            tags = eval(col1)
            for k, v in tags.items():
                self.audio_tags.AppendText('%s: %s\n' % (k, v))
        elif col0 == 'disposition':
            dispos = eval(col1)
            for k, v in dispos.items():
                self.audio_tags.AppendText('%s: %s\n' % (k, v))
        else:
            self.audio_tags.AppendText('%s: %s\n' % (col0, col1))
    # ------------------------------------------------------------------#

    def on_sub(self, event):
        """
        Shows the data of the selected item in the subtitle text box

        """
        if not self.button_view.GetValue():
            self.button_view.SetValue(True)
            self.on_view(self)
        item = self.subtitle_ctrl.GetFocusedItem()
        col0 = self.subtitle_ctrl.GetItemText(item, col=0)
        col1 = self.subtitle_ctrl.GetItemText(item, col=1)
        self.sub_tags.Clear()
        if col0 == 'tags':
            tags = eval(col1)
            for k, v in tags.items():
                self.sub_tags.AppendText('%s: %s\n' % (k, v))
        elif col0 == 'disposition':
            dispos = eval(col1)
            for k, v in dispos.items():
                self.sub_tags.AppendText('%s: %s\n' % (k, v))
        else:
            self.sub_tags.AppendText('%s: %s\n' % (col0, col1))
    # ------------------------------------------------------------------#

    def on_select(self, event):
        """
        Update and populate all checklists during items selection
        on self.file_select list control

        """
        # delete previous append:
        self.format_ctrl.DeleteAllItems()
        self.video_ctrl.DeleteAllItems()
        self.audio_ctrl.DeleteAllItems()
        self.subtitle_ctrl.DeleteAllItems()
        self.format_tags.Clear()
        self.video_tags.Clear()
        self.audio_tags.Clear()
        self.sub_tags.Clear()

        index = self.file_select.GetFocusedItem()
        item = self.file_select.GetItemText(index)

        index = 0

        for x in self.data:
            if x.get('format').get('filename') == item:
                select = self.data[self.data.index(x)]
                num_items = self.format_ctrl.GetItemCount()
                self.format_ctrl.InsertItem(num_items, 'DATA FORMAT:')
                self.format_ctrl.SetItemBackgroundColour(index, "ORANGE")
                index += 1
                for k, v in x.get('format').items():
                    self.format_ctrl.InsertItem(index, str(k))
                    self.format_ctrl.SetItem(index, 1, str(v))
                    index += 1

        if select.get('streams'):
            index = 0
            for t in select.get('streams'):
                if t.get('codec_type') == 'video':
                    num_items = self.video_ctrl.GetItemCount()
                    n = 'VIDEO INDEX %d' % t.get('index')
                    self.video_ctrl.InsertItem(num_items, n)
                    self.video_ctrl.SetItemBackgroundColour(index,
                                                            "SLATE BLUE")
                    index += 1
                    for k, v in t.items():
                        self.video_ctrl.InsertItem(index, str(k))
                        self.video_ctrl.SetItem(index, 1, str(v))
                        index += 1
            index = 0
            for t in select.get('streams'):
                if t.get('codec_type') == 'audio':
                    num_items = self.audio_ctrl.GetItemCount()
                    n = 'AUDIO INDEX %d' % t.get('index')
                    self.audio_ctrl.InsertItem(num_items, n)
                    self.audio_ctrl.SetItemBackgroundColour(index, "GREEN")
                    index += 1
                    for k, v in t.items():
                        self.audio_ctrl.InsertItem(index, str(k))
                        self.audio_ctrl.SetItem(index, 1, str(v))
                        index += 1
            index = 0
            for t in select.get('streams'):
                if t.get('codec_type') == 'subtitle':
                    num_items = self.subtitle_ctrl.GetItemCount()
                    n = 'SUBTITLE INDEX %d' % t.get('index')
                    self.subtitle_ctrl.InsertItem(num_items, n)
                    self.subtitle_ctrl.SetItemBackgroundColour(index,
                                                               "GOLDENROD")
                    index += 1
                    for k, v in t.items():
                        self.subtitle_ctrl.InsertItem(index, str(k))
                        self.subtitle_ctrl.SetItem(index, 1, str(v))
                        index += 1
    # ------------------------------------------------------------------#

    def on_close(self, event):
        self.Destroy()



# -*- coding: UTF-8 -*-
# Name: opendir.py
# Porpose: open file browser on given pathname
# Compatibility: Python3 (Unix, Windows)
# Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
# Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
# license: GPL3
# Rev: April.18.2021 *PEP8 compatible*
#########################################################

# This file is part of Videomass.

#    Videomass is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    Videomass is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

#########################################################
import subprocess
import os


def browse(OS, pathname):
    """
    open file browser in a specific location (OS independent)

    """
    status = 'Unrecognized error'

    if OS == 'Windows':
        try:
            os.startfile(os.path.realpath(pathname))

        except FileNotFoundError as pathnotfound:
            return '%s' % pathnotfound

        except Exception as anyerr:
            return '%s' % anyerr

        return None

    elif OS == 'Darwin':
        cmd = ['open', pathname]

    else:  # xdg-open *should* be supported by recent Gnome, KDE, Xfce
        cmd = ['xdg-open', pathname]

    try:
        p = subprocess.Popen(cmd,
                             stdout=subprocess.PIPE,
                             stderr=subprocess.STDOUT,
                             universal_newlines=True,  # mod text
                             )
        out = p.communicate()

    except (OSError, FileNotFoundError) as oserr:  # exec. do not exist
        status = '%s' % oserr
    else:
        if p.returncode:  # if returncode == 1
            status = out[0]
        else:
            status = None
    return status

    """
    NOTE The following code work, but on MS-Windows it show a short of
         Dos-window
    -----------------

    try:
        p = subprocess.run(cmd)
        if p.stderr:
            return(p.stderr.decode())
            '''
            if not *capture_output=True* on subprocess instance
            use .decode() here.
            '''
    except FileNotFoundError as err:
        return('%s' % (err))
    """
